#include <proto/exec.h>
#include <clib/alib_protos.h>
#include <devices/serial.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#define DEVICE_NAME "serial.device"
#define UNIT_NUMBER 0

#include "vecram.h" 
/* Amiga implementation of the vecram serial interface. */

static struct MsgPort *SerialMP;
static struct IOExtSer *SerialIO;

static int DeviceError;
static int CleanedUp=0;

void signalhandler (int sig)
{
	fprintf(stderr,"**BREAK**\n");
	exit (1);
}

void vecserial_init()
{

	if (atexit(vecserial_cleanup))
	{
		exit(1);
	}

        signal (SIGINT, signalhandler);

	SerialMP = CreatePort(NULL,0);
	if (SerialMP==NULL) { fprintf(stderr,"Couldn't create Msg Port\n"); exit(1); }
	SerialIO = (struct IOExtSer *) CreateExtIO(SerialMP, sizeof (struct IOExtSer));
	if (SerialIO==NULL) { fprintf(stderr,"Couldn't create IO Request Block\n"); exit(1); }
	DeviceError=OpenDevice (DEVICE_NAME,UNIT_NUMBER,(struct IORequest *) SerialIO,0);
	if (DeviceError!=0) { fprintf(stderr,"Couldn't open Serial Device\n"); exit(1); }

	SerialIO->IOSer.io_Command=SDCMD_SETPARAMS;
	SerialIO->io_Baud=38400;
	SerialIO->io_ReadLen=8;
	SerialIO->io_WriteLen=8;
	SerialIO->io_StopBits=1;
	SerialIO->io_SerFlags=SERF_RAD_BOOGIE|SERF_XDISABLED;

	DoIO((struct IORequest *)SerialIO);

}

void vecserial_cleanup()
{
	if (CleanedUp) return;
	if (!DeviceError) { CloseDevice((struct IORequest *) SerialIO); }
	if (SerialIO) { DeleteExtIO((struct IORequest *) SerialIO); }
	if (SerialMP) { DeletePort(SerialMP); }
	CleanedUp=1;
}

/* Read a single byte from the vecram serial */
unsigned char vecserial_read()
{
	unsigned char inchar;
	LONG WaitMask=SIGBREAKF_CTRL_C | 1L << SerialMP->mp_SigBit;
	LONG ResultMask;

	SerialIO->IOSer.io_Command=CMD_READ;
	SerialIO->IOSer.io_Length=1;
	SerialIO->IOSer.io_Data=&inchar;
	SendIO((struct IORequest *)SerialIO);
	
	while (1)
	{
	ResultMask=Wait(WaitMask);
		if (SIGBREAKF_CTRL_C & ResultMask)
		{
			AbortIO((struct IORequest *) SerialIO);
			WaitIO((struct IORequest *) SerialIO);
			printf("**BREAK**\n");
			exit(1);
		}
		if (CheckIO((struct IORequest *) SerialIO))
		{
			WaitIO((struct IORequest *) SerialIO);
			break;
		}	
	}
	return inchar;
}

void vecserial_write(unsigned char *buf, int numbytes)
{

	SerialIO->IOSer.io_Command=CMD_WRITE;
	SerialIO->IOSer.io_Length=numbytes;
	SerialIO->IOSer.io_Data=buf;

	DoIO((struct IORequest *)SerialIO);

}
