#include <stdio.h>
#include <termios.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include "vecram.h" 
#include <dzcomm.h>
/* DZCOM / DOS implementation of the vecram serial interface. */

/* Note that implementations of this interface must
 * Do error handling and ensure that any OS specific cleanup
 * gets done in the event of an error before they call exit. */

inline static void dz_print_comm_err() {printf("%s\n",szDZCommErr);fflush(stdout);}

static comm_port *port;

void vecserial_init()
{
	dzcomm_init();
	port = comm_port_init(_com1);
	if (port == NULL) {
      		dz_print_comm_err();
      		fprintf(stderr,"Error initialising port\n");
		exit(1);
	}
	// Set baud rate etc 38400 8n1
	comm_port_set_baud_rate(port,_38400);
	comm_port_set_data_bits(port,BITS_8);
	comm_port_set_stop_bits(port,STOP_1);
	comm_port_set_parity(port,NO_PARITY);

	if (!comm_port_install_handler(port)) {
       		dz_print_comm_err();
       		exit(1);
    	}
}

void vecserial_cleanup()
{
	if (port != NULL) 
		comm_port_uninstall(port);
	/* dzcomm_closedown(); Makes it crash */
}

/* Read a single byte from the vecram serial */
unsigned char vecserial_read()
{
	int res;
	do {
		res = comm_port_test(port);
	} while (res == -1);
	return (unsigned char) data_(res);
}

void vecserial_write(unsigned char *buf, int numbytes)
{
	int i;
	for (i=0; i<numbytes; i++) 
	{
		comm_port_out(port, buf[i]);
	}
}
