#include <stdio.h>
#include <termios.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include "vecram.h" 
/* Linux implementation of the vecram serial interface. */

/* Note that implementations of this interface must
 * Do error handling and ensure that any OS specific cleanup
 * gets done in the event of an error before they call exit.
 * The Linux implementation doesn't need to do this as Linux will
 * close the resources automatically. */

#define SERIAL_DEVICE "/dev/tts/0"

static int serialfd;

void vecserial_init()
{
	int res;
	struct termios serialparams;
	char sysbuf[300];
	sprintf(sysbuf,"/bin/setserial %s low_latency", SERIAL_DEVICE);
	res = system(sysbuf);
	if (res != 0) {
		fprintf(stderr,"Warning: unable to set low_latency, transfer will take much longer. (make sure /bin/setserial is installed)\n");
	}
	serialfd = open(SERIAL_DEVICE,O_RDWR );
	if (serialfd == -1)
	{
		perror("Opening serial device");
		exit(1);
	}
	tcgetattr(serialfd,&serialparams);
	cfsetispeed(&serialparams, B38400);
	cfsetospeed(&serialparams, B38400);
	serialparams.c_cflag &= ~PARENB; /* Disable parity */
	serialparams.c_cflag &= ~CSTOPB; /* Disable parity */
	serialparams.c_cflag |= CS8;
	serialparams.c_cflag |= (CLOCAL | CREAD); 
	cfmakeraw(&serialparams); /* Sets up raw mode */
	serialparams.c_cflag &= ~CRTSCTS; /* Disable hw flow control */
	res =tcsetattr(serialfd, TCSANOW, &serialparams);
	if (res != 0) {
		perror("setting speed");
		exit(1);
	}
}

void vecserial_cleanup()
{
	close(serialfd);
}

/* Read a single byte from the vecram serial */
unsigned char vecserial_read()
{
	unsigned char inchar;
	int res;
	res = read(serialfd, &inchar,1);
	if (res != 1) {
		perror("Reading");
		exit(1);
	}
	return inchar;
}

void vecserial_write(unsigned char *buf, int numbytes)
{
	int res = write(serialfd, buf, numbytes);
	if (res != numbytes) {
		perror("Writing");
		exit(1);
	}
}
