#include <windows.h>
#include <stdio.h>

// This is a handle to the comms port.
static HANDLE hCom;

void vecserial_init()
{
   DCB dcb;
   BOOL fSuccess;
   char *pcCommPort = "COM1";

   hCom = CreateFile( pcCommPort,
                    GENERIC_READ | GENERIC_WRITE,
                    0,    // must be opened with exclusive-access
                    NULL, // no security attributes
                    OPEN_EXISTING, // must use OPEN_EXISTING
                    0,    // not overlapped I/O
                    NULL  // hTemplate must be NULL for comm devices
                    );

   if (hCom == INVALID_HANDLE_VALUE) 
   {
       // Handle the error.
       printf ("CreateFile failed with error %ld.\n", GetLastError());
       exit(1);
   }

   // Build on the current configuration, and skip setting the size
   // of the input and output buffers with SetupComm.

   fSuccess = GetCommState(hCom, &dcb);

   if (!fSuccess) 
   {
      // Handle the error.
      printf ("GetCommState failed with error %ld.\n", GetLastError());
      exit(2);
   }

   // Fill in DCB: 38400 bps, 8 data bits, no parity, and 1 stop bit.

   dcb.BaudRate = CBR_38400;     // set the baud rate
   dcb.ByteSize = 8;             // data size, xmit, and rcv
   dcb.Parity = NOPARITY;        // no parity bit
   dcb.StopBits = ONESTOPBIT;    // one stop bit

   fSuccess = SetCommState(hCom, &dcb);

   if (!fSuccess) 
   {
      // Handle the error.
      printf ("SetCommState failed with error %ld.\n", GetLastError());
      exit(3);
   }

   printf ("Serial port %s successfully reconfigured.\n", pcCommPort);
}

void vecserial_cleanup()
{
	CloseHandle(hCom);
}

/*BOOL ReadFile(
  HANDLE hFile,
  LPVOID lpBuffer,
  DWORD nNumberOfBytesToRead,
  LPDWORD lpNumberOfBytesRead,
  LPOVERLAPPED lpOverlapped
);*/

unsigned char vecserial_read()
{
	unsigned char buf;
	DWORD bytesread;
	BOOL res;
	res = ReadFile(hCom, &buf, 1, &bytesread, NULL);
	if (!res) {
		printf("ReadFile failed with error %ld\n", GetLastError());
		vecserial_cleanup();
		exit(4);
	}
	return buf;
}

/*
BOOL WriteFile(
  HANDLE hFile,
  LPCVOID lpBuffer,
  DWORD nNumberOfBytesToWrite,
  LPDWORD lpNumberOfBytesWritten,
  LPOVERLAPPED lpOverlapped
);
*/

void vecserial_write(unsigned char *buf, int numbytes)
{
	DWORD byteswritten;
	BOOL res = WriteFile(hCom, buf, numbytes, &byteswritten, NULL);
	if ((!res) || (byteswritten != numbytes)) {
		printf("WriteFile failed with error %ld\n", GetLastError());
		vecserial_cleanup();
		exit(4);
	}
}
