#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "vecram.h"

#define MAXSIZE 32768

static int flag_quick=0; /* Use the n command instead of v */

static void waitforpoweron()
{
	unsigned char inchar;
	inchar = vecserial_read();
	printf("Read character %c\n",inchar);
	if (inchar != 'r') {
		printf("This probably isn't right\n");
	}
}

static void sendrom(char *fn) {
	unsigned char buf[MAXSIZE];
	FILE *f = fopen(fn,"rb");
	int filesize;
	int i;
	if (!f) {
		perror("Opening file\n");
		vecserial_cleanup();
		exit(1);
	}
	filesize = fread(buf,1, MAXSIZE, f); 
	fclose(f);
	for (i=0; i<filesize; i++) {
		unsigned char sendbuf[2];
		unsigned char retchar;
		if (flag_quick)
			sendbuf[0] = 'n';
		else
			sendbuf[0] = 'v';
		sendbuf[1] = buf[i];
		vecserial_write(sendbuf, 2);
		if (! flag_quick)
		{
			retchar = vecserial_read();
			if (buf[i] != retchar) {
				fprintf(stderr, "Data error at byte %04x\n",i);
				vecserial_cleanup();
				exit(1);
			}
		} else {
			/* Flag_quick */
			/* Send a load of NULs */
			unsigned char nulbuf[48];
			int j;
			for (j=0; j<48; j++) { nulbuf[j] = 'a'; }
			vecserial_write(nulbuf,8);
		}
		if ((i% 0x80) ==0)
		{
			printf("%04x Done\r",i);
		}
		fflush(stdout);
	}
	usleep(50000); /* 50ms */
	vecserial_write("c",1);
	printf("Finished.\n");
}

static void checkfile(char *fn) {
	FILE *f = fopen(fn,"r");
	if (! f) {
		/* No need to clean up ports - not opened yet */
		perror("Opening file");
		exit(1);
	}
	fclose(f);
}

int main(int argc, char *argv[]) {
	char *filename;
	int i;
	if (argc <2) {
		fprintf(stderr,"Not enough params\n");
		exit(1);
	}
	for (i=1; i<(argc-1); i++) {
		/* Check for flags */
		if (strcmp(argv[i],"-q") == 0) 
			flag_quick=1;
	}
	filename = argv[argc-1];

	checkfile(filename); /* Check file before doing stuff with serial ports */
	vecserial_init();
	waitforpoweron();
	sendrom(filename);
	vecserial_cleanup();
	return 0;
}

